#pragma once

#include "ofMain.h"
#include "AtlasLayerCombo.h"
#include "MsdfAtlasLayerCombo.h"
#include "MsdfLayer.h"
#include "GPUFontAtlasLayerCombo.h"
#include "GPUFontLayer.h"
#include "Utils.h"

namespace ofxVariableLab {

class LayerComposition {
    public:
        void setup();
        void update();
        void draw(int width = 0, int height = 0) const;
        void drawCamera(const ofCamera & camera) const;
        LayerID addLayer(const Layer::Props & props,
                         LayerID layerID = "");
        LayerID addLayer(const ComboIdentifier & identifier,
                         const string & text,
                         const std::vector <FontVariation> & variations,
                         AtlasLayerComboSettings & s,
                         LayerID layerID = "");
        LayerID addLayer(const ComboIdentifier & identifier,
                         const Layer::Props & props,
                         const std::vector <FontVariation> & variations,
                         AtlasLayerComboSettings & s,
                         LayerID layerID = "");
        void findOrCreateNewMomForLayer(shared_ptr <Layer> layer,
                                        ComboIdentifier idealMom);
        void removeLayer(const LayerID & id); // TODO: make bool, to catch nonexisting
        const std::unordered_map <LayerID, shared_ptr <Layer> > & getLayers();
        shared_ptr <Layer> getLayer(const LayerID & layerID);
        const std::unordered_map <ComboIdentifier, shared_ptr <AtlasLayerCombo> > & getAtlasLayerCombos() const;
        void setVFlip(bool vFlip);
        void setLayerOrder(const vector <LayerID> & layerOrder);

        [[deprecated("Replaced by setUniform1fv which also covers GPUFont")]]
        void setMsdfUniform1fv(const vector <string> & names,
                               const vector <float> & values);
        void setUniform1fv(const vector <string> & names,
                           const vector <float> & values);
        void setUniform1iv(const vector <string> & names,
                           const vector <int> & values);
        const LayerID getNextFreeLayerID() const;
    private:
        VFlipState vFlipState = V_FLIP_UNKNOWN;
        std::unordered_map <LayerID, shared_ptr <Layer> > layers;
        std::unordered_map <LayerID, shared_ptr <AtlasLayerCombo> > layerIDcombos;
        std::unordered_map <ComboIdentifier, shared_ptr <AtlasLayerCombo> > atlasLayerCombos;
        vector <LayerID> layerOrder;

        //std::unordered_map <LayerIdentifier, std::shared_ptr <ofxVariableLab::Layer> > layers;
};

}
