#pragma once

#include "AtlasLayerCombo.h"
#include "ofMain.h"
#include "ofxMsdfgen.h"
#include "Atlas.h"
#include "Layer.h"
#include "Utils.h"

namespace ofxVariableLab {

class MsdfLayer : public Layer {
    public:
        struct UboParameters {
            glm::vec4 fontColor;
            glm::vec4 bgColor;
            glm::vec2 unitRange;
            glm::vec2 translation_a;
            glm::vec2 scale_a;
            glm::vec2 translation_b;
            glm::vec2 scale_b;
            float mix;
        };
        struct BufferVertex {
            GLfloat position[4];
            GLfloat uv_a[2];
            GLfloat uv_b[2];
            GLfloat color[4];
            GLfloat mix;
            glm::mat4 globalTransform;
        };
        struct BufferBufferPrimitive {
            float w;
            float h;
            glm::vec2 translation_a;
            glm::vec2 scale_a;
            glm::vec2 translation_b;
            glm::vec2 scale_b;
        };
        void setup(const LayerSettings & settings = LayerSettings()) override;
        void update() override;
        void draw(glm::vec3 position = glm::vec3(0, 0, 0)) override;
        void drawCharacter(const char character,
                           glm::vec3 position = glm::vec3(0, 0, 0),
                           glm::vec4 color = glm::vec4(1, 1, 1, 1),
                           float rotation = 0,
                           float scale = 1,
                           ofxVariableLab::FontVariation fontVariation = ofxVariableLab::FontVariation()) override;
        void setProps(const Props & props) override;
        const Props & getProps(float delay = 0) override;
        void clearPropsBuffer() override;
        void setId(const LayerID & id) override;
        const LayerID & getId() override;
        void setShader(shared_ptr <ofShader> _shader) override;
        shared_ptr <ofShader> getShader() const override;
        const LayerType & getType() const override;
        void setVFlip(const VFlipState vFlip) override;
        bool isDirtyDirty() const override;
        void setDirtyDirty(bool dirtyDirty) override;
        bool wantsNewMom() override;
        void isWithNewMom() override;
        const ComboIdentifier & getMomsComboIdentifier() const override;
        void setMomsComboIdentifier(const ComboIdentifier & identifier) override;
        ofNode & getOuterNode() override;
        ofNode & getInnerNode() override;
        const Layer::BoundingBox & getBoundingBox() override;
        void setBoundingBox(const Layer::BoundingBox & boundingBox) override;
        bool collectCharacter(const char character,
                              vector <ofNode> & nodes,
                              const ofCamera & camera,
                              const vector <glm::vec4> & colors,
                              ofxVariableLab::FontVariation fontVariation = ofxVariableLab::FontVariation());

        void setAtlas(shared_ptr <ofxMsdfgen::Atlas> _atlas);
        shared_ptr <ofxMsdfgen::Atlas> getAtlas() const;

        shared_ptr <ofxMsdfgen::Atlas> atlas;

        LayerSettings settings;
        shared_ptr <ofShader> shader;
        /// \brief are props updated but not drawn yet
        bool isDirty = true;
        /// \brief hashed id, or just counting up
        string id = "";
        VFlipState vFlip = V_FLIP_UNKNOWN;
        float scaleFactor = 1.0;
        GLuint uboName;
        GLint uboIndex;
        UboParameters uboParameters;

        std::vector <BufferVertex> vertices;
        std::vector <int32_t> indices;
        int nElements = 0;

    private:
        Props lastProps;
        std::deque <Props> propsBuffer;
        bool notHappyWithMom = false;
        ComboIdentifier momsComboIdentifier;
        LayerType type = MSDFGEN;
        ofNode outerNode;
        ofNode innerNode;
        Layer::BoundingBox boundingBox;
        std::mutex vertices_mutex;
        std::mutex indices_mutex;
};
}
